@@ -9,14 +9,14 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
 """
@@ -27,10 +27,12 @@ a person.
 
 #-------------------------------------------------------------------------
 #
-# GRAMPS libraries
+# Gramps libraries
 #
 #-------------------------------------------------------------------------
 from .proxybase import ProxyDbBase
+from ..lib import (Person, Family, Source, Citation, Event, Media,
+                   Place, Repository, Note, Tag)
 
 class ReferencedBySelectionProxyDb(ProxyDbBase):
     """
@@ -42,7 +44,7 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
 
     def __init__(self, dbase, all_people=False):
         """
-        Create a new ReferencedByPeopleProxyDb instance.  
+        Create a new ReferencedByPeopleProxyDb instance.
 
         :param all_people: if True, get all people, and the items they link to;
                            if False, get all people that are connected to
@@ -60,107 +62,28 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         self.queue = []
         if all_people:
             # Do not add references to those not already included
-            self.restricted_to["Person"] = [x for x in 
+            self.restricted_to["Person"] = [x for x in
                                             self.db.iter_person_handles()]
             # Spread activation to all other items:
             for handle in self.restricted_to["Person"]:
-                self.queue_object("Person", handle)
+                if handle:
+                    self.queue_object("Person", handle)
         else:
             # get rid of orphaned people:
             # first, get all of the links from people:
             for person in self.db.iter_people():
-                self.queue_object("Person", person, False)
+                self.queue_object("Person", person.handle, False)
             # save those people:
             self.restricted_to["Person"] = self.referenced["Person"]
             # reset, and just follow those people
             self.reset_references()
             for handle in self.restricted_to["Person"]:
-                self.queue_object("Person", handle)
+                if handle:
+                    self.queue_object("Person", handle)
         # process:
         while len(self.queue):
             obj_type, handle, reference = self.queue.pop()
             self.process_object(obj_type, handle, reference)
-        # need to update _tables with these functions
-        self._tables['Person'].update(
-            {
-                "handle_func": self.get_person_from_handle, 
-                "gramps_id_func": self.get_person_from_gramps_id,
-                "class_func": Person,
-                "cursor_func": self.get_person_cursor,
-                "handles_func": self.get_person_handles,
-            })
-        self._tables['Family'].update(
-            {
-                "handle_func": self.get_family_from_handle, 
-                "gramps_id_func": self.get_family_from_gramps_id,
-                "class_func": Family,
-                "cursor_func": self.get_family_cursor,
-                "handles_func": self.get_family_handles,
-            })
-        self._tables['Source'].update(
-            {
-                "handle_func": self.get_source_from_handle, 
-                "gramps_id_func": self.get_source_from_gramps_id,
-                "class_func": Source,
-                "cursor_func": self.get_source_cursor,
-                "handles_func": self.get_source_handles,
-                })
-        self._tables['Citation'].update(
-            {
-                "handle_func": self.get_citation_from_handle, 
-                "gramps_id_func": self.get_citation_from_gramps_id,
-                "class_func": Citation,
-                "cursor_func": self.get_citation_cursor,
-                "handles_func": self.get_citation_handles,
-            })
-        self._tables['Event'].update(
-            {
-                "handle_func": self.get_event_from_handle, 
-                "gramps_id_func": self.get_event_from_gramps_id,
-                "class_func": Event,
-                "cursor_func": self.get_event_cursor,
-                "handles_func": self.get_event_handles,
-            })
-        self._tables['Media'].update(
-            {
-                "handle_func": self.get_object_from_handle, 
-                "gramps_id_func": self.get_object_from_gramps_id,
-                "class_func": MediaObject,
-                "cursor_func": self.get_media_cursor,
-                "handles_func": self.get_media_object_handles,
-            })
-        self._tables['Place'].update(
-            {
-                "handle_func": self.get_place_from_handle, 
-                "gramps_id_func": self.get_place_from_gramps_id,
-                "class_func": Place,
-                "cursor_func": self.get_place_cursor,
-                "handles_func": self.get_place_handles,
-            })
-        self._tables['Repository'].update(
-            {
-                "handle_func": self.get_repository_from_handle, 
-                "gramps_id_func": self.get_repository_from_gramps_id,
-                "class_func": Repository,
-                "cursor_func": self.get_repository_cursor,
-                "handles_func": self.get_repository_handles,
-            })
-        self._tables['Note'].update(
-            {
-                "handle_func": self.get_note_from_handle, 
-                "gramps_id_func": self.get_note_from_gramps_id,
-                "class_func": Note,
-                "cursor_func": self.get_note_cursor,
-                "handles_func": self.get_note_handles,
-            })
-        self._tables['Tag'].update(
-            {
-                "handle_func": self.get_tag_from_handle, 
-                "gramps_id_func": None,
-                "class_func": Tag,
-                "cursor_func": self.get_tag_cursor,
-                "handles_func": self.get_tag_handles,
-            })
 
     def queue_object(self, obj_type, handle, reference=True):
         self.queue.append((obj_type, handle, reference))
@@ -174,7 +97,7 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
             "Source": set(),
             "Citation": set(),
             "Repository": set(),
-            "MediaObject": set(),
+            "Media": set(),
             "Note": set(),
             "Tag": set(),
             }
@@ -208,8 +131,8 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
             obj = self.db.get_repository_from_handle(handle)
             if obj:
                 self.process_repository(obj)
-        elif class_name == "MediaObject":
-            obj = self.db.get_object_from_handle(handle)
+        elif class_name == "Media":
+            obj = self.db.get_media_from_handle(handle)
             if obj:
                 self.process_media(obj)
         elif class_name == "Note":
@@ -231,7 +154,7 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         if person.handle in self.referenced["Person"]:
             return
         # A person that we should not add:
-        if (self.restricted_to["Person"] and 
+        if (self.restricted_to["Person"] and
             person.handle not in self.restricted_to["Person"]):
             return
         if reference:
@@ -241,7 +164,7 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         for (class_name, handle) in self.db.find_backlink_handles(
             person.handle, ["Person", "Family"]):
             self.queue_object(class_name, handle)
-                
+
         name = person.get_primary_name()
         if name:
             self.process_name(name)
@@ -285,8 +208,10 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
             return
         self.referenced["Family"].add(family.handle)
 
-        self.queue_object("Person", family.mother_handle)
-        self.queue_object("Person", family.father_handle)
+        if family.mother_handle:
+            self.queue_object("Person", family.mother_handle)
+        if family.father_handle:
+            self.queue_object("Person", family.father_handle)
         for child_ref in family.get_child_ref_list():
             if not child_ref:
                 continue
@@ -305,6 +230,7 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         self.process_media_ref_list(family)
         self.process_attributes(family)
         self.process_lds_ords(family)
+        self.process_tags(family)
 
     def process_event(self, event):
         """
@@ -324,7 +250,9 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
             place = self.db.get_place_from_handle(place_handle)
             if place:
                 self.process_place(place)
-    
+
+        self.process_tags(event)
+
     def process_place(self, place):
         """
         Follow the place object and find all of the primary objects
@@ -338,6 +266,13 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         self.process_media_ref_list(place)
         self.process_urls(place)
 
+        for placeref in place.get_placeref_list():
+            place = self.db.get_place_from_handle(placeref.ref)
+            if place:
+                self.process_place(place)
+
+        self.process_tags(place)
+
     def process_source(self, source):
         """
         Follow the source object and find all of the primary objects
@@ -349,12 +284,13 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         for repo_ref in source.get_reporef_list():
             if repo_ref:
                 self.process_notes(repo_ref)
-                handle = repo_ref.get_reference_handle()                
+                handle = repo_ref.get_reference_handle()
                 repo = self.db.get_repository_from_handle(handle)
                 if repo:
                     self.process_repository(repo)
         self.process_media_ref_list(source)
         self.process_notes(source)
+        self.process_tags(source)
 
     def process_citation(self, citation):
         """
@@ -371,6 +307,7 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
                 self.process_source(source)
         self.process_media_ref_list(citation)
         self.process_notes(citation)
+        self.process_tags(citation)
 
     def process_repository(self, repository):
         """
@@ -383,18 +320,20 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         self.process_notes(repository)
         self.process_addresses(repository)
         self.process_urls(repository)
+        self.process_tags(repository)
 
     def process_media(self, media):
         """
         Follow the media object and find all of the primary objects
         that it references.
         """
-        if media is None or media.handle in self.referenced["MediaObject"]:
+        if media is None or media.handle in self.referenced["Media"]:
             return
-        self.referenced["MediaObject"].add(media.handle)
+        self.referenced["Media"].add(media.handle)
         self.process_citation_ref_list(media)
         self.process_attributes(media)
         self.process_notes(media)
+        self.process_tags(media)
 
     def process_note(self, note):
         """
@@ -409,9 +348,10 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
                 if tag.value.startswith("gramps://"):
                     obj_class, prop, value = tag.value[9:].split("/")
                     if obj_class == "Media":         # bug6493
-                        obj_class = "MediaObject"
+                        obj_class = "Media"
                     if prop == "handle":
                         self.queue_object(obj_class, value)
+        self.process_tags(note)
 
     def process_notes(self, original_obj):
         """ Find all of the primary objects referred to """
@@ -435,7 +375,7 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         """ Find all of the primary objects referred to """
         self.process_citation_ref_list(name)
         self.process_notes(name)
-    
+
     def process_addresses(self, original_obj):
         """ Find all of the primary objects referred to """
         for address in original_obj.get_address_list():
@@ -474,9 +414,9 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
                 self.process_attributes(media_ref)
                 self.process_citation_ref_list(media_ref)
                 handle = media_ref.get_reference_handle()
-                media_object = self.db.get_object_from_handle(handle)
-                if media_object:
-                    self.process_media(media_object)
+                media = self.db.get_media_from_handle(handle)
+                if media:
+                    self.process_media(media)
 
     def process_lds_ords(self, original_obj):
         """ Find all of the primary objects referred to """
@@ -487,9 +427,10 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
     def process_lds_ord(self, lds_ord):
         """ Find all of the primary objects referred to """
         fam_handle = lds_ord.get_family_handle()
-        fam = self.db.get_family_from_handle(fam_handle)
-        if fam:
-            self.queue_object("Family", fam_handle)
+        if fam_handle:
+            fam = self.db.get_family_from_handle(fam_handle)
+            if fam:
+                self.queue_object("Family", fam_handle)
 
         place_handle = lds_ord.get_place_handle()
         if place_handle:
@@ -525,25 +466,25 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         Filter for person
         """
         return handle in self.referenced["Person"]
-        
+
     def include_place(self, handle):
         """
         Filter for places
         """
         return handle in self.referenced["Place"]
-        
+
     def include_family(self, handle):
         """
         Filter for families
         """
         return handle in self.referenced["Family"]
-        
-    def include_media_object(self, handle):
+
+    def include_media(self, handle):
         """
         Filter for media objects
         """
-        return handle in self.referenced["MediaObject"]
-        
+        return handle in self.referenced["Media"]
+
     def include_event(self, handle):
         """
         Filter for events
@@ -555,36 +496,36 @@ class ReferencedBySelectionProxyDb(ProxyDbBase):
         Filter for sources
         """
         return handle in self.referenced["Source"]
-    
+
     def include_citation(self, handle):
         """
         Filter for citations
         """
         return handle in self.referenced["Citation"]
-    
+
     def include_repository(self, handle):
         """
         Filter for repositories
         """
         return handle in self.referenced["Repository"]
-        
+
     def include_note(self, handle):
         """
         Filter for notes
         """
         return handle in self.referenced["Note"]
-    
+
     def include_tag(self, handle):
         """
         Filter for tags
         """
         return handle in self.referenced["Tag"]
-    
+
     def find_backlink_handles(self, handle, include_classes=None):
         """
         Return appropriate backlink handles for this proxy.
         """
-        for (objclass, handle) in self.db.find_backlink_handles(handle, 
+        for (objclass, handle) in self.db.find_backlink_handles(handle,
                                                         include_classes):
             if handle in self.referenced[objclass]:
                 yield (objclass, handle)
