@@ -10,7 +10,7 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
@@ -52,33 +52,33 @@ _ = glocale.translation.sgettext
 # cache values; use refresh_constants() if they change
 try:
     from ..config import config
-    _MAX_AGE_PROB_ALIVE   = config.get('behavior.max-age-prob-alive')
-    _MAX_SIB_AGE_DIFF     = config.get('behavior.max-sib-age-diff')
-    _AVG_GENERATION_GAP   = config.get('behavior.avg-generation-gap')
+    _MAX_AGE_PROB_ALIVE = config.get('behavior.max-age-prob-alive')
+    _MAX_SIB_AGE_DIFF = config.get('behavior.max-sib-age-diff')
+    _AVG_GENERATION_GAP = config.get('behavior.avg-generation-gap')
 except ImportError:
     # Utils used as module not part of GRAMPS
-    _MAX_AGE_PROB_ALIVE   = 110
-    _MAX_SIB_AGE_DIFF     = 20
-    _AVG_GENERATION_GAP   = 20
+    _MAX_AGE_PROB_ALIVE = 110
+    _MAX_SIB_AGE_DIFF = 20
+    _AVG_GENERATION_GAP = 20
 
 #-------------------------------------------------------------------------
 #
 # ProbablyAlive class
 #
 #-------------------------------------------------------------------------
-class ProbablyAlive(object):
+class ProbablyAlive:
     """
     An object to hold the parameters for considering someone alive.
     """
 
-    def __init__(self, 
+    def __init__(self,
                  db,
-                 max_sib_age_diff=None, 
-                 max_age_prob_alive=None, 
+                 max_sib_age_diff=None,
+                 max_age_prob_alive=None,
                  avg_generation_gap=None):
         self.db = db
         if max_sib_age_diff is None:
-            max_sib_age_diff = _MAX_SIB_AGE_DIFF 
+            max_sib_age_diff = _MAX_SIB_AGE_DIFF
         if max_age_prob_alive is None:
             max_age_prob_alive = _MAX_AGE_PROB_ALIVE
         if avg_generation_gap is None:
@@ -86,6 +86,7 @@ class ProbablyAlive(object):
         self.MAX_SIB_AGE_DIFF = max_sib_age_diff
         self.MAX_AGE_PROB_ALIVE = max_age_prob_alive
         self.AVG_GENERATION_GAP = avg_generation_gap
+        self.pset = set()
 
     def probably_alive_range(self, person, is_spouse=False):
         # FIXME: some of these computed dates need to be a span. For
@@ -95,6 +96,7 @@ class ProbablyAlive(object):
         #        "between 1930 and 1940")
         if person is None:
             return (None, None, "", None)
+        self.pset = set()
         birth_ref = person.get_birth_ref()
         death_ref = person.get_death_ref()
         death_date = None
@@ -106,11 +108,7 @@ class ProbablyAlive(object):
             if death_ref:
                 death = self.db.get_event_from_handle(death_ref.ref)
                 if death:
-                    if death.get_date_object().is_valid():
-                        death_date = death.get_date_object()
-                    else: # has a death event, but it is not valid:
-                        death_date = Today() # before today
-                        death_date.set_modifier(Date.MOD_BEFORE)
+                    death_date = death.get_date_object()
 
         # Look for Cause Of Death, Burial or Cremation events.
         # These are fairly good indications that someone's not alive.
@@ -142,19 +140,22 @@ class ProbablyAlive(object):
 
         if not birth_date and death_date:
             # person died more than MAX after current year
-            birth_date = death_date.copy_offset_ymd(year=-self.MAX_AGE_PROB_ALIVE)
+            if death_date.is_valid():
+                birth_date = death_date.copy_offset_ymd(year=-self.MAX_AGE_PROB_ALIVE)
+            else:
+                birth_date = death_date
             explain = _("death date")
-        
+
         if not death_date and birth_date:
             # person died more than MAX after current year
             death_date = birth_date.copy_offset_ymd(year=self.MAX_AGE_PROB_ALIVE)
             explain = _("birth date")
-        
+
         if death_date and birth_date:
             return (birth_date, death_date, explain, person) # direct self evidence
-        
+
         # Neither birth nor death events are available. Try looking
-        # at siblings. If a sibling was born more than X years past, 
+        # at siblings. If a sibling was born more than X years past,
         # or more than Z future, then probably this person is
         # not alive. If the sibling died more than X years
         # past, or more than X years future, then probably not alive.
@@ -173,7 +174,7 @@ class ProbablyAlive(object):
                 for ev_ref in child.get_primary_event_ref_list():
                     ev = self.db.get_event_from_handle(ev_ref.ref)
                     if ev and ev.type.is_birth():
-                        dobj = ev.get_date_object() 
+                        dobj = ev.get_date_object()
                         if dobj.get_start_date() != Date.EMPTY:
                             # if sibling birth date too far away, then not alive:
                             year = dobj.get_year()
@@ -184,14 +185,14 @@ class ProbablyAlive(object):
                                         _("sibling birth date"),
                                         child)
                     elif ev and ev.type.is_death():
-                        dobj = ev.get_date_object() 
+                        dobj = ev.get_date_object()
                         if dobj.get_start_date() != Date.EMPTY:
                             # if sibling death date too far away, then not alive:
                             year = dobj.get_year()
                             if year != 0:
                                 # sibling death date
                                 return (Date().copy_ymd(year - self.MAX_SIB_AGE_DIFF - self.MAX_AGE_PROB_ALIVE),
-                                        Date().copy_ymd(year - self.MAX_SIB_AGE_DIFF - self.MAX_AGE_PROB_ALIVE 
+                                        Date().copy_ymd(year - self.MAX_SIB_AGE_DIFF - self.MAX_AGE_PROB_ALIVE
                                                                 + self.MAX_AGE_PROB_ALIVE),
                                         _("sibling death date"),
                                         child)
@@ -199,7 +200,7 @@ class ProbablyAlive(object):
                 for ev_ref in child.get_primary_event_ref_list():
                     ev = self.db.get_event_from_handle(ev_ref.ref)
                     if ev and ev.type.is_birth_fallback():
-                        dobj = ev.get_date_object() 
+                        dobj = ev.get_date_object()
                         if dobj.get_start_date() != Date.EMPTY:
                             # if sibling birth date too far away, then not alive:
                             year = dobj.get_year()
@@ -210,7 +211,7 @@ class ProbablyAlive(object):
                                         _("sibling birth-related date"),
                                         child)
                     elif ev and ev.type.is_death_fallback():
-                        dobj = ev.get_date_object() 
+                        dobj = ev.get_date_object()
                         if dobj.get_start_date() != Date.EMPTY:
                             # if sibling death date too far away, then not alive:
                             year = dobj.get_year()
@@ -258,12 +259,12 @@ class ProbablyAlive(object):
                                 year = date.get_year()
                                 if year != 0:
                                     other = None
-                                    if person.handle == mother_handle and father_handle: 
+                                    if person.handle == mother_handle and father_handle:
                                         other = self.db.get_person_from_handle(father_handle)
-                                    elif person.handle == father_handle and mother_handle: 
+                                    elif person.handle == father_handle and mother_handle:
                                         other = self.db.get_person_from_handle(mother_handle)
                                     return (Date().copy_ymd(year - self.AVG_GENERATION_GAP),
-                                            Date().copy_ymd(year - self.AVG_GENERATION_GAP + 
+                                            Date().copy_ymd(year - self.AVG_GENERATION_GAP +
                                                                     self.MAX_AGE_PROB_ALIVE),
 
                                             _("event with spouse"), other)
@@ -272,9 +273,12 @@ class ProbablyAlive(object):
         # ago.
 
         def descendants_too_old (person, years):
+            if person.handle in self.pset:
+                return (None, None, "", None)
+            self.pset.add(person.handle)
             for family_handle in person.get_family_handle_list():
                 family = self.db.get_family_from_handle(family_handle)
-                if not family: 
+                if not family:
                     # can happen with LivingProxyDb(PrivateProxyDb(db))
                     continue
                 for child_ref in family.get_child_ref_list():
@@ -297,7 +301,7 @@ class ProbablyAlive(object):
                         child_death = self.db.get_event_from_handle(child_death_ref.ref)
                         dobj = child_death.get_date_object()
                         if dobj.get_start_date() != Date.EMPTY:
-                            return (dobj.copy_offset_ymd(- self.AVG_GENERATION_GAP), 
+                            return (dobj.copy_offset_ymd(- self.AVG_GENERATION_GAP),
                                     dobj.copy_offset_ymd(- self.AVG_GENERATION_GAP + self.MAX_AGE_PROB_ALIVE),
                                     _("descendant death date"),
                                     child)
@@ -308,7 +312,7 @@ class ProbablyAlive(object):
                     for ev_ref in child.get_primary_event_ref_list():
                         ev = self.db.get_event_from_handle(ev_ref.ref)
                         if ev and ev.type.is_birth_fallback():
-                            dobj = ev.get_date_object() 
+                            dobj = ev.get_date_object()
                             if dobj.get_start_date() != Date.EMPTY:
                                 d = Date(dobj)
                                 val = d.get_start_date()
@@ -319,9 +323,9 @@ class ProbablyAlive(object):
                                         child)
 
                         elif ev and ev.type.is_death_fallback():
-                            dobj = ev.get_date_object() 
+                            dobj = ev.get_date_object()
                             if dobj.get_start_date() != Date.EMPTY:
-                                return (dobj.copy_offset_ymd(- self.AVG_GENERATION_GAP), 
+                                return (dobj.copy_offset_ymd(- self.AVG_GENERATION_GAP),
                                         dobj.copy_offset_ymd(- self.AVG_GENERATION_GAP + self.MAX_AGE_PROB_ALIVE),
                                         _("descendant death-related date"),
                                         child)
@@ -343,12 +347,15 @@ class ProbablyAlive(object):
             return (date1, date2, explain, other)
 
         def ancestors_too_old(person, year):
+            if person.handle in self.pset:
+                return (None, None, "", None)
+            self.pset.add(person.handle)
             LOG.debug("ancestors_too_old('%s', %s)".format(
                 name_displayer.display(person), year) )
             family_handle = person.get_main_parents_family_handle()
-            if family_handle:                
+            if family_handle:
                 family = self.db.get_family_from_handle(family_handle)
-                if not family: 
+                if not family:
                     # can happen with LivingProxyDb(PrivateProxyDb(db))
                     return (None, None, "", None)
                 father_handle = family.get_father_handle()
@@ -360,7 +367,7 @@ class ProbablyAlive(object):
                             father_birth_ref.ref)
                         dobj = father_birth.get_date_object()
                         if dobj.get_start_date() != Date.EMPTY:
-                            return (dobj.copy_offset_ymd(- year), 
+                            return (dobj.copy_offset_ymd(- year),
                                     dobj.copy_offset_ymd(- year + self.MAX_AGE_PROB_ALIVE),
                                     _("ancestor birth date"),
                                     father)
@@ -370,7 +377,7 @@ class ProbablyAlive(object):
                             father_death_ref.ref)
                         dobj = father_death.get_date_object()
                         if dobj.get_start_date() != Date.EMPTY:
-                            return (dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE), 
+                            return (dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE),
                                     dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE + self.MAX_AGE_PROB_ALIVE),
                                     _("ancestor death date"),
                                     father)
@@ -379,17 +386,17 @@ class ProbablyAlive(object):
                     for ev_ref in father.get_primary_event_ref_list():
                         ev = self.db.get_event_from_handle(ev_ref.ref)
                         if ev and ev.type.is_birth_fallback():
-                            dobj = ev.get_date_object() 
+                            dobj = ev.get_date_object()
                             if dobj.get_start_date() != Date.EMPTY:
-                                return (dobj.copy_offset_ymd(- year), 
+                                return (dobj.copy_offset_ymd(- year),
                                         dobj.copy_offset_ymd(- year + self.MAX_AGE_PROB_ALIVE),
                                         _("ancestor birth-related date"),
                                         father)
 
                         elif ev and ev.type.is_death_fallback():
-                            dobj = ev.get_date_object() 
+                            dobj = ev.get_date_object()
                             if dobj.get_start_date() != Date.EMPTY:
-                                return (dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE), 
+                                return (dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE),
                                         dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE + self.MAX_AGE_PROB_ALIVE),
                                         _("ancestor death-related date"),
                                         father)
@@ -406,7 +413,7 @@ class ProbablyAlive(object):
                         mother_birth = self.db.get_event_from_handle(mother_birth_ref.ref)
                         dobj = mother_birth.get_date_object()
                         if dobj.get_start_date() != Date.EMPTY:
-                            return (dobj.copy_offset_ymd(- year), 
+                            return (dobj.copy_offset_ymd(- year),
                                     dobj.copy_offset_ymd(- year + self.MAX_AGE_PROB_ALIVE),
                                     _("ancestor birth date"),
                                     mother)
@@ -416,7 +423,7 @@ class ProbablyAlive(object):
                             mother_death_ref.ref)
                         dobj = mother_death.get_date_object()
                         if dobj.get_start_date() != Date.EMPTY:
-                            return (dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE), 
+                            return (dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE),
                                     dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE + self.MAX_AGE_PROB_ALIVE),
                                     _("ancestor death date"),
                                     mother)
@@ -425,17 +432,17 @@ class ProbablyAlive(object):
                     for ev_ref in mother.get_primary_event_ref_list():
                         ev = self.db.get_event_from_handle(ev_ref.ref)
                         if ev and ev.type.is_birth_fallback():
-                            dobj = ev.get_date_object() 
+                            dobj = ev.get_date_object()
                             if dobj.get_start_date() != Date.EMPTY:
-                                return (dobj.copy_offset_ymd(- year), 
+                                return (dobj.copy_offset_ymd(- year),
                                         dobj.copy_offset_ymd(- year + self.MAX_AGE_PROB_ALIVE),
                                         _("ancestor birth-related date"),
                                         mother)
 
                         elif ev and ev.type.is_death_fallback():
-                            dobj = ev.get_date_object() 
+                            dobj = ev.get_date_object()
                             if dobj.get_start_date() != Date.EMPTY:
-                                return (dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE), 
+                                return (dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE),
                                         dobj.copy_offset_ymd(- year - self.MAX_AGE_PROB_ALIVE + self.MAX_AGE_PROB_ALIVE),
                                         _("ancestor death-related date"),
                                         mother)
@@ -451,7 +458,7 @@ class ProbablyAlive(object):
             # then assume our person must be dead too.
             date1, date2, explain, other = ancestors_too_old (person, - self.AVG_GENERATION_GAP)
         except RuntimeError:
-            raise Errors.DatabaseError(
+            raise DatabaseError(
                 _("Database error: loop in %s's ancestors") %
                 name_displayer.display(person))
         if date1 and date2:
@@ -467,11 +474,11 @@ class ProbablyAlive(object):
 # probably_alive
 #
 #-------------------------------------------------------------------------
-def probably_alive(person, db, 
-                   current_date=None, 
+def probably_alive(person, db,
+                   current_date=None,
                    limit=0,
-                   max_sib_age_diff=None, 
-                   max_age_prob_alive=None, 
+                   max_sib_age_diff=None,
+                   max_age_prob_alive=None,
                    avg_generation_gap=None,
                    return_range=False):
     """
@@ -499,22 +506,22 @@ def probably_alive(person, db,
         birth, death, explain))
     if not birth or not death:
         # no evidence, must consider alive
-        return ((True, None, None, _("no evidence"), None) if return_range 
+        return ((True, None, None, _("no evidence"), None) if return_range
                 else True)
     # must have dates from here:
     if limit:
         death += limit # add these years to death
     # Finally, check to see if current_date is between dates
-    result = (current_date.match(birth, ">=") and 
+    result = (current_date.match(birth, ">=") and
               current_date.match(death, "<="))
     if return_range:
         return (result, birth, death, explain, relative)
     else:
         return result
 
-def probably_alive_range(person, db, 
-                         max_sib_age_diff=None, 
-                         max_age_prob_alive=None, 
+def probably_alive_range(person, db,
+                         max_sib_age_diff=None,
+                         max_age_prob_alive=None,
                          avg_generation_gap=None):
     """
     Computes estimated birth and death dates.
@@ -527,7 +534,7 @@ def probably_alive_range(person, db,
     while isinstance(basedb, ProxyDbBase):
         basedb = basedb.db
     # Now, we create a wrapper for doing work:
-    pb = ProbablyAlive(basedb, max_sib_age_diff, 
+    pb = ProbablyAlive(basedb, max_sib_age_diff,
                        max_age_prob_alive, avg_generation_gap)
     return pb.probably_alive_range(person)
 
@@ -537,6 +544,6 @@ def update_constants():
     """
     from ..config import config
     global _MAX_AGE_PROB_ALIVE, _MAX_SIB_AGE_DIFF, _AVG_GENERATION_GAP
-    _MAX_AGE_PROB_ALIVE   = config.get('behavior.max-age-prob-alive')
-    _MAX_SIB_AGE_DIFF     = config.get('behavior.max-sib-age-diff')
-    _AVG_GENERATION_GAP   = config.get('behavior.avg-generation-gap')
+    _MAX_AGE_PROB_ALIVE = config.get('behavior.max-age-prob-alive')
+    _MAX_SIB_AGE_DIFF = config.get('behavior.max-sib-age-diff')
+    _AVG_GENERATION_GAP = config.get('behavior.avg-generation-gap')
