@@ -25,7 +25,7 @@
 """
 Polish-specific classes for parsing and displaying dates.
 """
-from __future__ import unicode_literals
+
 #-------------------------------------------------------------------------
 #
 # Python modules
@@ -35,7 +35,7 @@ import re
 
 #-------------------------------------------------------------------------
 #
-# GRAMPS modules
+# Gramps modules
 #
 #-------------------------------------------------------------------------
 from ..lib.date import Date
@@ -54,38 +54,38 @@ class DateParserPL(DateParser):
     month_to_int["styczeń"] = 1
     month_to_int["sty"] = 1
     month_to_int["I"] = 1
-    month_to_int["luty"]  = 2
-    month_to_int["lut"]  = 2
-    month_to_int["II"]  = 2
-    month_to_int["marzec"]  = 3
-    month_to_int["mar"]  = 3
-    month_to_int["III"]  = 3
-    month_to_int["kwiecień"]  = 4
-    month_to_int["kwi"]  = 4
-    month_to_int["IV"]  = 4
-    month_to_int["maj"]  = 5
-    month_to_int["V"]  = 5
-    month_to_int["czerwiec"]  = 6
-    month_to_int["cze"]  = 6
-    month_to_int["VI"]  = 6
-    month_to_int["lipiec"]  = 7
-    month_to_int["lip"]  = 7
-    month_to_int["VII"]  = 7
-    month_to_int["sierpień"]  = 8
-    month_to_int["sie"]  = 8
-    month_to_int["VIII"]  = 8
-    month_to_int["wrzesień"]  = 9
-    month_to_int["wrz"]  = 9
-    month_to_int["IX"]  = 9
-    month_to_int["październik"]  = 10
-    month_to_int["paź"]  = 10
-    month_to_int["X"]  = 10
-    month_to_int["listopad"]  = 11
-    month_to_int["lis"]  = 11
-    month_to_int["XI"]  = 11
-    month_to_int["grudzień"]  = 12
-    month_to_int["gru"]  = 12
-    month_to_int["XII"]  = 12
+    month_to_int["luty"] = 2
+    month_to_int["lut"] = 2
+    month_to_int["II"] = 2
+    month_to_int["marzec"] = 3
+    month_to_int["mar"] = 3
+    month_to_int["III"] = 3
+    month_to_int["kwiecień"] = 4
+    month_to_int["kwi"] = 4
+    month_to_int["IV"] = 4
+    month_to_int["maj"] = 5
+    month_to_int["V"] = 5
+    month_to_int["czerwiec"] = 6
+    month_to_int["cze"] = 6
+    month_to_int["VI"] = 6
+    month_to_int["lipiec"] = 7
+    month_to_int["lip"] = 7
+    month_to_int["VII"] = 7
+    month_to_int["sierpień"] = 8
+    month_to_int["sie"] = 8
+    month_to_int["VIII"] = 8
+    month_to_int["wrzesień"] = 9
+    month_to_int["wrz"] = 9
+    month_to_int["IX"] = 9
+    month_to_int["październik"] = 10
+    month_to_int["paź"] = 10
+    month_to_int["X"] = 10
+    month_to_int["listopad"] = 11
+    month_to_int["lis"] = 11
+    month_to_int["XI"] = 11
+    month_to_int["grudzień"] = 12
+    month_to_int["gru"] = 12
+    month_to_int["XII"] = 12
     # Alternative forms: declined nouns
     month_to_int["stycznia"] = 1
     month_to_int["lutego"] = 2
@@ -112,13 +112,13 @@ class DateParserPL(DateParser):
     month_to_int["paz"] = 10
 
     modifier_to_int = {
-        'przed'    : Date.MOD_BEFORE, 
+        'przed'    : Date.MOD_BEFORE,
         'po'   : Date.MOD_AFTER,
         'około'  : Date.MOD_ABOUT,
         'ok.'     : Date.MOD_ABOUT,
         'circa'  : Date.MOD_ABOUT,
         'ca.'  : Date.MOD_ABOUT,
-    	# Alternative forms: misspellings sometimes used in emails
+        # Alternative forms: misspellings sometimes used in emails
         'okolo'  : Date.MOD_ABOUT,
         'ok'     : Date.MOD_ABOUT,
         }
@@ -135,7 +135,7 @@ class DateParserPL(DateParser):
         'francuski republikański': Date.CAL_FRENCH,
         'franc.'         : Date.CAL_FRENCH,
         'perski'         : Date.CAL_PERSIAN,
-        'szwedzki'       : Date.CAL_SWEDISH, 
+        'szwedzki'       : Date.CAL_SWEDISH,
         's'              : Date.CAL_SWEDISH,
     # Alternative forms: nouns without polish accent letters
     # (misspellings sometimes used in emails)
@@ -153,16 +153,19 @@ class DateParserPL(DateParser):
 
     bce = ["przed naszą erą", "przed Chrystusem",
            "p.n.e."] + DateParser.bce
-    
+
     def init_strings(self):
         DateParser.init_strings(self)
-        self._span  = re.compile("(od)\s+(?P<start>.+)\s+(do)\s+(?P<stop>.+)", re.IGNORECASE)
+        self._span = re.compile(
+            r"(od)\s+(?P<start>.+)\s+(do)\s+(?P<stop>.+)", re.IGNORECASE)
         # Also handle a common mistakes
-        self._range = re.compile("((?:po)?mi(?:ę|e)dzy)\s+(?P<start>.+)\s+(a)\s+(?P<stop>.+)", re.IGNORECASE)
-        self._text2 = re.compile('(\d+)?.?\s+?%s\s*((\d+)(/\d+)?)?' % self._mon_str,
-                                 re.IGNORECASE)
-        self._jtext2 = re.compile('(\d+)?.?\s+?%s\s*((\d+)(/\d+)?)?' % self._jmon_str,
-                                 re.IGNORECASE)
+        self._range = re.compile(
+            r"((?:po)?mi(?:ę|e)dzy)\s+(?P<start>.+)\s+(a)\s+(?P<stop>.+)",
+            re.IGNORECASE)
+        self._text2 = re.compile(r'(\d+)?.?\s+?%s\s*((\d+)(/\d+)?)?'
+                                 % self._mon_str, re.IGNORECASE)
+        self._jtext2 = re.compile(r'(\d+)?.?\s+?%s\s*((\d+)(/\d+)?)?'
+                                  % self._jmon_str, re.IGNORECASE)
 
 #-------------------------------------------------------------------------
 #
@@ -171,34 +174,34 @@ class DateParserPL(DateParser):
 #-------------------------------------------------------------------------
 class DateDisplayPL(DateDisplay):
     """
-    Polish language date display class. 
+    Polish language date display class.
     """
 
-    long_months = ( "", "Styczeń", "Luty", "Marzec", "Kwiecień", "Maj", 
-                    "Czerwiec", "Lipiec", "Sierpień", "Wrzesień", "Październik", 
+    long_months = ( "", "Styczeń", "Luty", "Marzec", "Kwiecień", "Maj",
+                    "Czerwiec", "Lipiec", "Sierpień", "Wrzesień", "Październik",
                     "Listopad", "Grudzień" )
-    
-    short_months = ( "", "Sty", "Lut", "Mar", "Kwi", "Maj", "Cze", 
+
+    short_months = ( "", "Sty", "Lut", "Mar", "Kwi", "Maj", "Cze",
                      "Lip", "Sie", "Wrz", "Paź", "Lis", "Gru" )
-    
+
     calendar = (
-        "", "juliański", "hebrajski", 
-        "francuski republikański", "perski", "islamski", 
-        "swedish" 
+        "", "juliański", "hebrajski",
+        "francuski republikański", "perski", "islamski",
+        "swedish"
         )
 
     _mod_str = ("", "przed ", "po ", "ok. ", "", "", "")
-    
+
     _qual_str = ("", "szacowany ", "obliczony ")
-    
+
     _bce_str = "%s p.n.e."
 
     formats = (
         "RRRR-MM-DD (ISO)", "Numeryczny", "Miesiąc Dzień, Rok",
-        "Dzień.Miesiąc.Rok", "Dzień Miesiąc Rok", "Dzień MieRzym Rok"
+        "Miesiąc.Dzień.Rok", "Dzień Miesiąc Rok", "Dzień MieRzym Rok"
         )
         # this definition must agree with its "_display_gregorian" method
-    
+
     roman_months = (
         "",
         "I",
@@ -215,7 +218,7 @@ class DateDisplayPL(DateDisplay):
         "XII"
         )
 
-    def _display_gregorian(self, date_val):
+    def _display_gregorian(self, date_val, **kwargs):
         """
         display gregorian calendar date in different format
         """
@@ -224,15 +227,15 @@ class DateDisplayPL(DateDisplay):
         if self.format == 0:
             return self.display_iso(date_val)
         elif self.format == 1:
-            # month_number.day.year
+            # day.month_number.year
             if date_val[3]:
                 return self.display_iso(date_val)
             else:
                 if date_val[0] == date_val[1] == 0:
                     value = str(date_val[2])
                 else:
-                    value = self._tformat.replace('%m', str(date_val[0]))
-                    value = value.replace('%d', str(date_val[1]))
+                    value = self.dhformat.replace('%d', str(date_val[0]))
+                    value = value.replace('%m', str(date_val[1]))
                     value = value.replace('%Y', str(date_val[2]))
         elif self.format == 2:
             # month_name day, year
@@ -242,17 +245,17 @@ class DateDisplayPL(DateDisplay):
                 else:
                     value = "%s %s" % (self.long_months[date_val[1]], year)
             else:
-                value = "%s %d, %s" % (self.long_months[date_val[1]], 
+                value = "%s %d, %s" % (self.long_months[date_val[1]],
                                        date_val[0], year)
         elif self.format == 3:
-            # day. month_number. year
+            # month_number. day. year
             if date_val[0] == 0:
                 if date_val[1] == 0:
                     value = year
                 else:
                     value = "%d.%s" % (date_val[1], year)
             else:
-                value = "%d.%d.%s" % (date_val[0], date_val[1], year)
+                value = "%d.%d.%s" % (date_val[1], date_val[0], year)
         elif self.format == 4:
             # day month_name year
             if date_val[0] == 0:
@@ -261,7 +264,7 @@ class DateDisplayPL(DateDisplay):
                 else:
                     value = "%s %s" % (self.long_months[date_val[1]], year)
             else:
-                value = "%d %s %s" % (date_val[0], 
+                value = "%d %s %s" % (date_val[0],
                                       self.long_months[date_val[1]], year)
         else:
             # day Roman_number_month year
@@ -271,7 +274,7 @@ class DateDisplayPL(DateDisplay):
                 else:
                     value = "%s %s" % (self.roman_months[date_val[1]], year)
             else:
-                value = "%d %s %s" % (date_val[0], 
+                value = "%d %s %s" % (date_val[0],
                                       self.roman_months[date_val[1]], year)
         if date_val[2] < 0:
             return self._bce_str % value
@@ -289,7 +292,7 @@ class DateDisplayPL(DateDisplay):
         newyear = date.get_new_year()
 
         qual_str = self._qual_str[qual]
-        
+
         if mod == Date.MOD_TEXTONLY:
             return date.get_text()
         elif start == Date.EMPTY:
@@ -298,18 +301,18 @@ class DateDisplayPL(DateDisplay):
             d1 = self.display_cal[cal](start)
             d2 = self.display_cal[cal](date.get_stop_date())
             scal = self.format_extras(cal, newyear)
-            return "%s%s %s %s %s%s" % (qual_str, 'od', d1, 'do', d2, 
+            return "%s%s %s %s %s%s" % (qual_str, 'od', d1, 'do', d2,
                                         scal)
         elif mod == Date.MOD_RANGE:
             d1 = self.display_cal[cal](start)
             d2 = self.display_cal[cal](date.get_stop_date())
             scal = self.format_extras(cal, newyear)
-            return "%s%s %s %s %s%s" % (qual_str, 'między', d1, 'a', d2, 
+            return "%s%s %s %s %s%s" % (qual_str, 'między', d1, 'a', d2,
                                         scal)
         else:
             text = self.display_cal[date.get_calendar()](start)
             scal = self.format_extras(cal, newyear)
-            return "%s%s%s%s" % (qual_str, self._mod_str[mod], text, 
+            return "%s%s%s%s" % (qual_str, self._mod_str[mod], text,
                                  scal)
 
 #-------------------------------------------------------------------------
@@ -317,5 +320,6 @@ class DateDisplayPL(DateDisplay):
 # Register classes
 #
 #-------------------------------------------------------------------------
-register_datehandler(('pl_PL','polish','Polish_Poland','pl'),
-                     DateParserPL, DateDisplayPL)
+register_datehandler(
+    ('pl_PL', 'polish', 'Polish_Poland', 'pl', ('%d.%m.%Y',)),
+    DateParserPL, DateDisplayPL)
