@@ -38,7 +38,7 @@ from . import MergePersonQuery
 # MergeFamilyQuery
 #
 #-------------------------------------------------------------------------
-class MergeFamilyQuery(object):
+class MergeFamilyQuery:
     """
     Create database query to merge two families.
     """
@@ -133,21 +133,50 @@ class MergeFamilyQuery(object):
         old_handle = self.titanic.get_handle()
 
         with DbTxn(_('Merge Family'), self.database) as trans:
+            # commit family in case Phoenix GrampsID, relationship has changed
+            self.database.commit_family(self.phoenix, trans)
+            if self.phoenix_fh != self.titanic_fh:
+                if self.phoenix_fh:
+                    phoenix_father = self.database.get_person_from_handle(
+                        self.phoenix_fh)
+                else:
+                    phoenix_father = None
+                if self.titanic_fh:
+                    titanic_father = self.database.get_person_from_handle(
+                        self.titanic_fh)
+                else:
+                    titanic_father = None
+                self.merge_person(phoenix_father, titanic_father,
+                                  'father', trans)
 
-            phoenix_father = self.database.get_person_from_handle(self.phoenix_fh)
-            titanic_father = self.database.get_person_from_handle(self.titanic_fh)
-            self.merge_person(phoenix_father, titanic_father, 'father', trans)
-
-            phoenix_mother = self.database.get_person_from_handle(self.phoenix_mh)
-            titanic_mother = self.database.get_person_from_handle(self.titanic_mh)
+            if self.phoenix_mh != self.titanic_mh:
+                if self.phoenix_mh:
+                    phoenix_mother = self.database.get_person_from_handle(
+                        self.phoenix_mh)
+                else:
+                    phoenix_mother = None
+                if self.titanic_mh:
+                    titanic_mother = self.database.get_person_from_handle(
+                        self.titanic_mh)
+                else:
+                    titanic_mother = None
+                self.merge_person(phoenix_mother, titanic_mother,
+                                  'mother', trans)
+            # Reload families from db in case the merge_person above changed
+            # them
             self.phoenix = self.database.get_family_from_handle(new_handle)
             self.titanic = self.database.get_family_from_handle(old_handle)
-            self.merge_person(phoenix_mother, titanic_mother, 'mother', trans)
 
-            phoenix_father = self.database.get_person_from_handle(self.phoenix_fh)
-            phoenix_mother = self.database.get_person_from_handle(self.phoenix_mh)
-            self.phoenix = self.database.get_family_from_handle(new_handle)
-            self.titanic = self.database.get_family_from_handle(old_handle)
+            if self.phoenix_fh:
+                phoenix_father = self.database.get_person_from_handle(
+                    self.phoenix_fh)
+            else:
+                phoenix_father = None
+            if self.phoenix_mh:
+                phoenix_mother = self.database.get_person_from_handle(
+                    self.phoenix_mh)
+            else:
+                phoenix_mother = None
             self.phoenix.merge(self.titanic)
             self.database.commit_family(self.phoenix, trans)
             for childref in self.titanic.get_child_ref_list():
@@ -168,6 +197,8 @@ class MergeFamilyQuery(object):
             # replace the family in lds ordinances
             for (dummy, person_handle) in self.database.find_backlink_handles(
                     old_handle, ['Person']):
+                if person_handle in (self.titanic_fh, self.titanic_mh):
+                    continue
                 person = self.database.get_person_from_handle(person_handle)
                 person.replace_handle_reference('Family', old_handle,new_handle)
                 self.database.commit_person(person, trans)
