@@ -9,7 +9,7 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
@@ -48,28 +48,31 @@ from gi.repository import GObject
 
 #-------------------------------------------------------------------------
 #
-# GRAMPS modules
+# Gramps modules
 #
 #-------------------------------------------------------------------------
 from gramps.gen.const import GRAMPS_LOCALE as glocale
+from gramps.gen.db.dbconst import DBBACKEND
+from gramps.gen.db.utils import make_database
 _ = glocale.translation.gettext
 from gramps.cli.grampscli import CLIDbLoader
 from gramps.gen.config import config
-from gramps.gen.db import DbBsddb
-from gramps.gen.db.exceptions import (DbUpgradeRequiredError, 
-                                      BsddbDowngradeError, 
-                                      DbVersionError, 
+from gramps.gen.db.exceptions import (DbUpgradeRequiredError,
+                                      BsddbDowngradeError,
+                                      DbVersionError,
+                                      DbPythonError,
                                       DbEnvironmentError,
                                       BsddbUpgradeRequiredError,
                                       BsddbDowngradeRequiredError,
                                       PythonUpgradeRequiredError,
-                                      PythonDowngradeError)
-from gramps.gen.constfunc import STRTYPE, UNITYPE, conv_to_unicode
+                                      PythonDowngradeError,
+                                      DbConnectionError)
 from .pluginmanager import GuiPluginManager
-from .dialog import (DBErrorDialog, ErrorDialog, QuestionDialog2, 
+from .dialog import (DBErrorDialog, ErrorDialog, QuestionDialog2,
                             WarningDialog)
 from .user import User
 from gramps.gen.errors import DbError
+from .managedwindow import ManagedWindow
 
 #-------------------------------------------------------------------------
 #
@@ -81,39 +84,31 @@ class DbLoader(CLIDbLoader):
         CLIDbLoader.__init__(self, dbstate)
         self.uistate = uistate
         self.import_info = None
-    
+
     def _warn(self, title, warnmessage):
-        WarningDialog(title, warnmessage, parent=self.uistate.window)
-    
+        WarningDialog(title, warnmessage,
+                      parent=self.uistate.window)
+
     def _errordialog(self, title, errormessage):
         """
-        Show the error. 
+        Show the error.
         In the GUI, the error is shown, and a return happens
         """
-        ErrorDialog(title, errormessage, parent=self.uistate.window)
+        ErrorDialog(title, errormessage,
+                    parent=self.uistate.window)
         return 1
-    
+
     def _dberrordialog(self, msg):
         import traceback
         exc = traceback.format_exc()
         try:
-            DBErrorDialog(str(msg.value), parent=self.uistate.window)
+            DBErrorDialog(str(msg.value),
+                          parent=self.uistate.window)
             _LOG.error(str(msg.value))
         except:
-            DBErrorDialog(str(msg), parent=self.uistate.window)
+            DBErrorDialog(str(msg),
+                          parent=self.uistate.window)
             _LOG.error(str(msg) +"\n" + exc)
-    
-    def _begin_progress(self):
-        self.uistate.set_busy_cursor(True)
-        self.uistate.progress.show()
-        self.uistate.pulse_progressbar(0)
-    
-    def _pulse_progress(self, value):
-        self.uistate.pulse_progressbar(value)
-
-    def _end_progress(self):
-        self.uistate.set_busy_cursor(False)
-        self.uistate.progress.hide()
 
     def import_file(self):
         self.import_info = None
@@ -129,146 +124,16 @@ class DbLoader(CLIDbLoader):
                   'If you think you may want to revert the import, '
                   'please stop here and backup your database.'),
                 _('_Proceed with import'), _('_Stop'),
-                self.uistate.window)
+                parent=self.uistate.window)
             if not warn_dialog.run():
                 return False
-            
-        pmgr = GuiPluginManager.get_instance()
-        
-        import_dialog = Gtk.FileChooserDialog(_('Gramps: Import Family Tree'), 
-                                       self.uistate.window, 
-                                       Gtk.FileChooserAction.OPEN, 
-                                       (Gtk.STOCK_CANCEL, 
-                                            Gtk.ResponseType.CANCEL, 
-                                        _('Import'), 
-                                            Gtk.ResponseType.OK))
-        import_dialog.set_local_only(False)
 
-        # Always add automatic (match all files) filter
-        add_all_files_filter(import_dialog)   # *
+        GrampsImportFileDialog(self.dbstate, self.uistate,
+                               callback=self.set_info)
 
-        # Add more file type selections for available importers
-        for plugin in pmgr.get_import_plugins():
-            file_filter = Gtk.FileFilter()
-            name = "%s (.%s)" % (plugin.get_name(), plugin.get_extension())
-            file_filter.set_name(name)
-            file_filter.add_pattern("*.%s" % plugin.get_extension())
-            file_filter.add_pattern(plugin.get_extension().capitalize())
-            import_dialog.add_filter(file_filter)
+    def set_info(self, info):
+        self.import_info = info
 
-        (box, type_selector) = format_maker()
-        import_dialog.set_extra_widget(box)
-
-        # Suggested folder: try last open file, import, then last export, 
-        # then home.
-        default_dir = config.get('paths.recent-import-dir')
-        if len(default_dir)<=1:
-            default_dir = get_default_dir()
-
-        import_dialog.set_current_folder(default_dir)
-        while True:
-            response = import_dialog.run()
-            if response == Gtk.ResponseType.CANCEL:
-                break
-            elif response == Gtk.ResponseType.OK:
-                filename = conv_to_unicode(import_dialog.get_filename())
-                if self.check_errors(filename):
-                    # displays errors if any
-                    continue
-
-                (the_path, the_file) = os.path.split(filename)
-                config.set('paths.recent-import-dir', the_path)
-                
-                extension = type_selector.get_value()
-                if extension == 'auto':
-                    # Guess the file format based on the file extension.
-                    # This will get the lower case extension without a period, 
-                    # or an empty string.
-                    extension = os.path.splitext(filename)[-1][1:].lower()
-                
-                for plugin in pmgr.get_import_plugins():
-                    if extension == plugin.get_extension():
-                        self.do_import(import_dialog, 
-                                       plugin.get_import_function(), 
-                                       filename)
-                        return True
-
-                # Finally, we give up and declare this an unknown format
-                ErrorDialog(
-                    _("Could not open file: %s") % filename, 
-                    _('File type "%s" is unknown to Gramps.\n\n'
-                      'Valid types are: Gramps database, Gramps XML, '
-                      'Gramps package, GEDCOM, and others.') % extension,
-                    parent=self.uistate.window)
-
-        import_dialog.destroy()
-        return False
-
-    def check_errors(self, filename):
-        """
-        Run common error checks and return True if any found.
-        
-        In this process, a warning dialog can pop up.
-        
-        """
-        if not isinstance(filename, (STRTYPE, UNITYPE)):
-            return True
-
-        filename = os.path.normpath(os.path.abspath(filename))
-
-        if len(filename) == 0:
-            return True
-        elif os.path.isdir(filename):
-            ErrorDialog(
-                _('Cannot open file'), 
-                _('The selected file is a directory, not a file.\n'),
-                parent=self.uistate.window)
-            return True
-        elif os.path.exists(filename):
-            if not os.access(filename, os.R_OK):
-                ErrorDialog(
-                    _('Cannot open file'), 
-                    _('You do not have read access to the selected file.'),
-                    parent=self.uistate.window)
-                return True
-        else:
-            try:
-                f = file(filename,'w')
-                f.close()
-                os.remove(filename)
-            except IOError:
-                ErrorDialog(
-                    _('Cannot create file'),
-                    _('You do not have write access to the selected file.'),
-                    parent=self.uistate.window)
-                return True
-
-        return False
-
-    def do_import(self, dialog, importer, filename):
-        self.import_info = None
-        dialog.destroy()
-        self._begin_progress()
-
-        try:
-            #an importer can return an object with info, object.info_text() 
-            #returns that info. Otherwise None is set to import_info
-            self.import_info = importer(self.dbstate.db, filename,
-                            User(callback=self._pulse_progress,
-                                uistate=self.uistate))
-            dirname = os.path.dirname(filename) + os.path.sep
-            config.set('paths.recent-import-dir', dirname)
-        except UnicodeError as msg:
-            ErrorDialog(
-                _("Could not import file: %s") % filename, 
-                _("This file incorrectly identifies its character "
-                  "set, so it cannot be accurately imported. Please fix the "
-                  "encoding, and import again") + "\n\n %s" % msg,
-                parent=self.uistate.window)
-        except Exception:
-            _LOG.error("Failed to import database.", exc_info=True)
-        self._end_progress()
-    
     def import_info_text(self):
         """
         On import the importer can construct an info object about the import.
@@ -278,13 +143,13 @@ class DbLoader(CLIDbLoader):
         if self.import_info is None:
             return ""
         return self.import_info.info_text()
-    
-    def read_file(self, filename):
+
+    def read_file(self, filename, username=None, password=None):
         """
         This method takes care of changing database, and loading the data.
-        In 3.0 we only allow reading of real databases of filetype 
+        In 3.0 we only allow reading of real databases of filetype
         'x-directory/normal'
-        
+
         This method should only return on success.
         Returning on failure makes no sense, because we cannot recover,
         since database has already been changed.
@@ -297,7 +162,7 @@ class DbLoader(CLIDbLoader):
         if os.path.exists(filename):
             if not os.access(filename, os.W_OK):
                 mode = "r"
-                self._warn(_('Read only database'), 
+                self._warn(_('Read only database'),
                                              _('You do not have write access '
                                                'to the selected file.'))
             else:
@@ -305,12 +170,26 @@ class DbLoader(CLIDbLoader):
         else:
             mode = 'w'
 
-        db = DbBsddb()
+        dbid_path = os.path.join(filename, DBBACKEND)
+        if os.path.isfile(dbid_path):
+            with open(dbid_path) as fp:
+                dbid = fp.read().strip()
+        else:
+            dbid = "bsddb"
+
+        db = make_database(dbid)
         db.disable_signals()
         self.dbstate.no_database()
 
+        if db.requires_login() and username is None:
+            login = GrampsLoginDialog(self.uistate)
+            credentials = login.run()
+            if credentials is None:
+                return
+            username, password = credentials
+
         self._begin_progress()
-        
+
         force_schema_upgrade = False
         force_bsddb_upgrade = False
         force_bsddb_downgrade = False
@@ -318,20 +197,26 @@ class DbLoader(CLIDbLoader):
         try:
             while True:
                 try:
-                    db.load(filename, self._pulse_progress, 
+                    db.load(filename, self._pulse_progress,
                             mode, force_schema_upgrade,
                             force_bsddb_upgrade,
                             force_bsddb_downgrade,
-                            force_python_upgrade)
-                    db.set_save_path(filename)
+                            force_python_upgrade,
+                            username=username,
+                            password=password)
+                    if self.dbstate.is_open():
+                        self.dbstate.db.close(
+                            user=User(callback=self._pulse_progress,
+                                      uistate=self.uistate,
+                                      dbstate=self.dbstate))
                     self.dbstate.change_database(db)
                     break
                 except DbUpgradeRequiredError as msg:
-                    if QuestionDialog2(_("Are you sure you want to upgrade "
-                                         "this Family Tree?"), 
-                                       str(msg), 
+                    if QuestionDialog2(_("Are you sure you want "
+                                         "to upgrade this Family Tree?"),
+                                       str(msg),
                                        _("I have made a backup,\n"
-                                         "please upgrade my Family Tree"), 
+                                         "please upgrade my Family Tree"),
                                        _("Cancel"),
                                        parent=self.uistate.window).run():
                         force_schema_upgrade = True
@@ -342,11 +227,11 @@ class DbLoader(CLIDbLoader):
                         self.dbstate.no_database()
                         break
                 except BsddbUpgradeRequiredError as msg:
-                    if QuestionDialog2(_("Are you sure you want to upgrade "
-                                         "this Family Tree?"), 
-                                       str(msg), 
+                    if QuestionDialog2(_("Are you sure you want "
+                                         "to upgrade this Family Tree?"),
+                                       str(msg),
                                        _("I have made a backup,\n"
-                                         "please upgrade my tree"), 
+                                         "please upgrade my Family Tree"),
                                        _("Cancel"),
                                        parent=self.uistate.window).run():
                         force_schema_upgrade = False
@@ -357,11 +242,11 @@ class DbLoader(CLIDbLoader):
                         self.dbstate.no_database()
                         break
                 except BsddbDowngradeRequiredError as msg:
-                    if QuestionDialog2(_("Are you sure you want to downgrade "
-                                         "this Family Tree?"), 
-                                       str(msg), 
+                    if QuestionDialog2(_("Are you sure you want "
+                                         "to downgrade this Family Tree?"),
+                                       str(msg),
                                        _("I have made a backup,\n"
-                                         "please downgrade my Family Tree"), 
+                                         "please downgrade my Family Tree"),
                                        _("Cancel"),
                                        parent=self.uistate.window).run():
                         force_schema_upgrade = False
@@ -372,11 +257,11 @@ class DbLoader(CLIDbLoader):
                         self.dbstate.no_database()
                         break
                 except PythonUpgradeRequiredError as msg:
-                    if QuestionDialog2(_("Are you sure you want to upgrade "
-                                         "this Family Tree?"), 
-                                       str(msg), 
+                    if QuestionDialog2(_("Are you sure you want "
+                                         "to upgrade this Family Tree?"),
+                                       str(msg),
                                        _("I have made a backup,\n"
-                                         "please upgrade my Family Tree"), 
+                                         "please upgrade my Family Tree"),
                                        _("Cancel"),
                                        parent=self.uistate.window).run():
                         force_schema_upgrade = False
@@ -393,12 +278,18 @@ class DbLoader(CLIDbLoader):
         except DbVersionError as msg:
             self.dbstate.no_database()
             self._errordialog( _("Cannot open database"), str(msg))
+        except DbPythonError as msg:
+            self.dbstate.no_database()
+            self._errordialog( _("Cannot open database"), str(msg))
         except DbEnvironmentError as msg:
             self.dbstate.no_database()
             self._errordialog( _("Cannot open database"), str(msg))
         except PythonDowngradeError as msg:
             self.dbstate.no_database()
             self._warn( _("Cannot open database"), str(msg))
+        except DbConnectionError as msg:
+            self.dbstate.no_database()
+            self._warn(_("Cannot open database"), str(msg))
         except OSError as msg:
             self.dbstate.no_database()
             self._errordialog(
@@ -412,27 +303,6 @@ class DbLoader(CLIDbLoader):
         self._end_progress()
         return True
 
-#-------------------------------------------------------------------------
-#
-# default dir selection
-#
-#-------------------------------------------------------------------------
-def get_default_dir():
-    # Suggested folder: try last open file, last import, last export, 
-    # then home.
-    default_dir = os.path.dirname(config.get('paths.recent-file'))
-    if default_dir:
-        default_dir += os.path.sep
-        if len(default_dir)<=1:
-            default_dir = config.get('paths.recent-import-dir')
-        if len(default_dir)<=1:
-            default_dir = config.get('paths.recent-export-dir')
-        if len(default_dir)<=1:
-            default_dir = '~/'
-    else:
-        default_dir = "~/"
-    return default_dir
-
 #-------------------------------------------------------------------------
 #
 # FileChooser filters: what to show in the file chooser
@@ -447,6 +317,13 @@ def add_all_files_filter(chooser):
     mime_filter.add_pattern('*')
     chooser.add_filter(mime_filter)
 
+
+def icase(ext):
+    """
+    Return a glob reresenting a case insensitive file extension.
+    """
+    return ''.join(['[{}{}]'.format(s.lower(), s.upper()) for s in ext])
+
 #-------------------------------------------------------------------------
 #
 # Format selectors: explictly set the format of the file
@@ -455,7 +332,7 @@ def add_all_files_filter(chooser):
 class GrampsFormatWidget(Gtk.ComboBox):
 
     def __init__(self):
-        GObject.GObject.__init__(self, model=None)
+        Gtk.ComboBox.__init__(self, model=None)
 
     def set(self, format_list):
         self.store = Gtk.ListStore(GObject.TYPE_STRING)
@@ -464,7 +341,7 @@ class GrampsFormatWidget(Gtk.ComboBox):
         self.pack_start(cell, True)
         self.add_attribute(cell, 'text', 0)
         self.format_list = format_list
-        
+
         for format, label in format_list:
             self.store.append(row=[label])
         self.set_active(False)
@@ -478,21 +355,21 @@ class GrampsFormatWidget(Gtk.ComboBox):
 def format_maker():
     """
     A factory function making format selection widgets.
-    
+
     Accepts a list of formats to include into selector.
     The auto selection is always added as the first one.
     The returned box contains both the label and the selector.
     """
     pmgr = GuiPluginManager.get_instance()
     format_list = [ ('auto', _('Automatically detected')) ]
-    
+
     for plugin in pmgr.get_import_plugins():
         format_list.append( (plugin.get_extension(), plugin.get_name()) )
 
     type_selector = GrampsFormatWidget()
     type_selector.set(format_list)
 
-    box = Gtk.HBox()
+    box = Gtk.Box()
     label = Gtk.Label(label=_('Select file _type:'))
     label.set_use_underline(True)
     label.set_mnemonic_widget(type_selector)
@@ -500,3 +377,210 @@ def format_maker():
     box.add(type_selector)
     box.show_all()
     return (box, type_selector)
+
+class GrampsLoginDialog(ManagedWindow):
+
+    def __init__(self, uistate):
+        """
+        A login dialog to obtain credentials to connect to a database
+        """
+        self.title = _("Login")
+        ManagedWindow.__init__(self, uistate, [], self.__class__, modal=True)
+
+        dialog = Gtk.Dialog(parent=uistate.window)
+        grid = Gtk.Grid()
+        grid.set_border_width(6)
+        grid.set_row_spacing(6)
+        grid.set_column_spacing(6)
+        label = Gtk.Label(label=_('Username: '))
+        grid.attach(label, 0, 0, 1, 1)
+        self.username = Gtk.Entry()
+        self.username.set_hexpand(True)
+        grid.attach(self.username, 1, 0, 1, 1)
+        label = Gtk.Label(label=_('Password: '))
+        grid.attach(label, 0, 1, 1, 1)
+        self.password = Gtk.Entry()
+        self.password.set_hexpand(True)
+        self.password.set_visibility(False)
+        self.password.set_input_purpose(Gtk.InputPurpose.PASSWORD)
+        grid.attach(self.password, 1, 1, 1, 1)
+        dialog.vbox.pack_start(grid, True, True, 0)
+        dialog.add_buttons(_('_Cancel'), Gtk.ResponseType.CANCEL,
+                           _('Login'), Gtk.ResponseType.OK)
+        self.set_window(dialog, None, self.title)
+
+    def run(self):
+        self.show()
+        response = self.window.run()
+        username = self.username.get_text()
+        password = self.password.get_text()
+        if response == Gtk.ResponseType.CANCEL:
+            self.close()
+            return None
+        elif response == Gtk.ResponseType.OK:
+            self.close()
+            return (username, password)
+
+class GrampsImportFileDialog(ManagedWindow):
+
+    def __init__(self, dbstate, uistate, callback=None):
+        """
+        A dialog to import a file into Gramps
+        """
+        self.dbstate = dbstate
+
+        self.title = _("Import Family Tree")
+        ManagedWindow.__init__(self, uistate, [], self.__class__, modal=True)
+        # the import_dialog.run() below makes it modal, so any change to
+        # the previous line's "modal" would require that line to be changed
+
+        pmgr = GuiPluginManager.get_instance()
+
+        import_dialog = Gtk.FileChooserDialog(
+            title='', transient_for=self.uistate.window,
+            action=Gtk.FileChooserAction.OPEN)
+        import_dialog.add_buttons(_('_Cancel'), Gtk.ResponseType.CANCEL,
+                                  _('Import'), Gtk.ResponseType.OK)
+        self.set_window(import_dialog, None, self.title)
+        self.setup_configs('interface.grampsimportfiledialog', 780, 630)
+        import_dialog.set_local_only(False)
+
+        # Always add automatic (match all files) filter
+        add_all_files_filter(import_dialog)   # *
+
+        # Add more file type selections for available importers
+        for plugin in pmgr.get_import_plugins():
+            file_filter = Gtk.FileFilter()
+            name = "%s (.%s)" % (plugin.get_name(), plugin.get_extension())
+            file_filter.set_name(name)
+            file_filter.add_pattern("*.%s" % icase(plugin.get_extension()))
+            import_dialog.add_filter(file_filter)
+
+        (box, type_selector) = format_maker()
+        import_dialog.set_extra_widget(box)
+
+        import_dialog.set_current_folder(config.get('paths.recent-import-dir'))
+        while True:
+            # the import_dialog.run() makes it modal, so any change to that
+            # line would require the ManagedWindow.__init__ to be changed also
+            response = import_dialog.run()
+            if response == Gtk.ResponseType.CANCEL:
+                break
+            elif response == Gtk.ResponseType.DELETE_EVENT:
+                return
+            elif response == Gtk.ResponseType.OK:
+                filename = import_dialog.get_filename()
+                if self.check_errors(filename):
+                    # displays errors if any
+                    continue
+
+                (the_path, the_file) = os.path.split(filename)
+                config.set('paths.recent-import-dir', the_path)
+
+                extension = type_selector.get_value()
+                if extension == 'auto':
+                    # Guess the file format based on the file extension.
+                    # This will get the lower case extension without a period,
+                    # or an empty string.
+                    extension = os.path.splitext(filename)[-1][1:].lower()
+
+                for plugin in pmgr.get_import_plugins():
+                    if extension == plugin.get_extension():
+                        self.close()
+                        self.do_import(plugin.get_import_function(),
+                                       filename)
+                        if callback is not None:
+                            callback(self.import_info)
+                        return
+
+                # Finally, we give up and declare this an unknown format
+                ErrorDialog(
+                    _("Could not open file: %s") % filename,
+                    _('File type "%s" is unknown to Gramps.\n\n'
+                      'Valid types are: Gramps database, Gramps XML, '
+                      'Gramps package, GEDCOM, and others.') % extension,
+                    parent=self.uistate.window)
+
+        self.close()
+
+    def check_errors(self, filename):
+        """
+        Run common error checks and return True if any found.
+
+        In this process, a warning dialog can pop up.
+
+        """
+        if not isinstance(filename, str):
+            return True
+
+        filename = os.path.normpath(os.path.abspath(filename))
+
+        if len(filename) == 0:
+            return True
+        elif os.path.isdir(filename):
+            ErrorDialog(
+                _('Cannot open file'),
+                _('The selected file is a directory, not a file.\n'),
+                parent=self.uistate.window)
+            return True
+        elif os.path.exists(filename):
+            if not os.access(filename, os.R_OK):
+                ErrorDialog(
+                    _('Cannot open file'),
+                    _('You do not have read access to the selected file.'),
+                    parent=self.uistate.window)
+                return True
+        else:
+            try:
+                f = open(filename, 'w')
+                f.close()
+                os.remove(filename)
+            except IOError:
+                ErrorDialog(
+                    _('Cannot create file'),
+                    _('You do not have write access to the selected file.'),
+                    parent=self.uistate.window)
+                return True
+
+        return False
+
+    def do_import(self, importer, filename):
+        self.import_info = None
+        self._begin_progress()
+        self.uistate.set_sensitive(False)
+
+        try:
+            #an importer can return an object with info, object.info_text()
+            #returns that info. Otherwise None is set to import_info
+            self.import_info = importer(self.dbstate.db, filename,
+                            User(callback=self._pulse_progress,
+                                 uistate=self.uistate,
+                                 dbstate=self.dbstate))
+            dirname = os.path.dirname(filename) + os.path.sep
+            config.set('paths.recent-import-dir', dirname)
+        except UnicodeError as msg:
+            ErrorDialog(
+                _("Could not import file: %s") % filename,
+                _("This file incorrectly identifies its character "
+                  "set, so it cannot be accurately imported. Please fix the "
+                  "encoding, and import again") + "\n\n %s" % msg,
+                parent=self.uistate.window)
+        except Exception:
+            _LOG.error("Failed to import database.", exc_info=True)
+        self.uistate.set_sensitive(True)
+        self._end_progress()
+
+    def build_menu_names(self, obj): # this is meaningless since it's modal
+        return (self.title, None)
+
+    def _begin_progress(self):
+        self.uistate.set_busy_cursor(True)
+        self.uistate.progress.show()
+        self.uistate.pulse_progressbar(0)
+
+    def _pulse_progress(self, value):
+        self.uistate.pulse_progressbar(value)
+
+    def _end_progress(self):
+        self.uistate.set_busy_cursor(False)
+        self.uistate.progress.hide()
