@@ -36,27 +36,35 @@ import sys
 #-------------------------------------------------------------------------
 from gramps.gen import user
 from .utils import ProgressMeter
-from .dialog import (WarningDialog, ErrorDialog, DBErrorDialog, 
-                            InfoDialog, QuestionDialog2)
+from .dialog import (WarningDialog, ErrorDialog, DBErrorDialog,
+                     RunDatabaseRepair, InfoDialog, QuestionDialog2)
+
 #-------------------------------------------------------------------------
 #
 # User class
 #
 #-------------------------------------------------------------------------
-class User(user.User):
+class User(user.UserBase):
     """
     This class provides a means to interact with the user via GTK.
-    It implements the interface in :class:`.gen.user.User`
+    It implements the interface in :class:`.gen.user.UserBase`
     """
-    def __init__(self, callback=None, error=None, uistate=None):
-        user.User.__init__(self, callback, error)
+    def __init__(self, callback=None, error=None, parent=None,
+                 uistate=None, dbstate=None): # TODO User API: gen==cli==gui
+        user.UserBase.__init__(self, callback, error, uistate, dbstate)
         self._progress = None
-        self.uistate = uistate
-    
+
+        if parent:
+            self.parent = parent
+        elif uistate:
+            self.parent = uistate.window
+        else:
+            self.parent = None
+
     def begin_progress(self, title, message, steps):
         """
         Start showing a progress indicator to the user.
-        
+
         :param title: the title of the progress meter
         :type title: str
         :param message: the message associated with the progress meter
@@ -67,15 +75,12 @@ class User(user.User):
         :type steps: int
         :returns: none
         """
-        if self.uistate:
-            self._progress = ProgressMeter(title, parent=self.uistate.window)
-        else:
-            self._progress = ProgressMeter(title)
+        self._progress = ProgressMeter(title, parent=self.parent)
         if steps > 0:
             self._progress.set_pass(message, steps, ProgressMeter.MODE_FRACTION)
         else:
             self._progress.set_pass(message, mode=ProgressMeter.MODE_ACTIVITY)
-    
+
     def step_progress(self):
         """
         Advance the progress meter.
@@ -90,11 +95,12 @@ class User(user.User):
         if self._progress:
             self._progress.close()
             self._progress = None
-    
-    def prompt(self, title, message, accept_label, reject_label, parent=None):
+
+    def prompt(self, title, message, accept_label, reject_label, parent=None,
+               default_label=None):
         """
         Prompt the user with a message to select an alternative.
-        
+
         :param title: the title of the question, e.g.: "Undo history warning"
         :type title: str
         :param message: the message, e.g.: "Proceeding with the tool will erase
@@ -105,30 +111,33 @@ class User(user.User):
         :param accept_label: what to call the positive choice, e.g.: "Proceed"
         :type accept_label: str
         :param reject_label: what to call the negative choice, e.g.: "Stop"
+        :param default_label: the label of the default
+        :type default_label: str or None
         :type reject_label: str
         :returns: the user's answer to the question
         :rtype: bool
         """
-        dialog = QuestionDialog2(title, message, accept_label, reject_label,
-                                 parent)
+        dialog = QuestionDialog2(title, message,
+                                 accept_label, reject_label,
+                                 parent=parent)
         return dialog.run()
-    
+
     def warn(self, title, warning=""):
         """
         Warn the user.
-        
+
         :param title: the title of the warning
         :type title: str
         :param warning: the warning
         :type warning: str
         :returns: none
         """
-        WarningDialog(title, warning)
-    
+        WarningDialog(title, warning, parent=self.parent)
+
     def notify_error(self, title, error=""):
         """
         Notify the user of an error.
-        
+
         :param title: the title of the error
         :type title: str
         :param error: the error message
@@ -136,19 +145,29 @@ class User(user.User):
         :returns: none
         """
         if self.error_function:
-            self.error_function(title, error)
+            self.error_function(title, error, parent=self.parent)
         else:
-            ErrorDialog(title, error)
+            ErrorDialog(title, error, parent=self.parent)
 
     def notify_db_error(self, error):
         """
         Notify the user of a DB error.
-        
+
         :param error: the DB error message
         :type error: str
         :returns: none
         """
-        DBErrorDialog(error)
+        DBErrorDialog(error, parent=self.parent)
+
+    def notify_db_repair(self, error):
+        """
+        Notify the user their DB might need repair.
+
+        :param error: the DB error message
+        :type error: str
+        :returns: none
+        """
+        RunDatabaseRepair(error, parent=self.parent)
 
     def info(self, msg1, infotext, parent=None, monospaced=False):
         """
