@@ -16,17 +16,22 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+#------------------------------------------------------------------------
+#
+# Python modules
+#
+#------------------------------------------------------------------------
 from collections import defaultdict
 
 #------------------------------------------------------------------------
 #
-# GRAMPS modules
+# Gramps modules
 #
 #------------------------------------------------------------------------
 from gramps.gen.plug import Gramplet
+from gramps.gen.config import config
 from gramps.gen.const import GRAMPS_LOCALE as glocale
 _ = glocale.translation.sgettext
-from gramps.gen.config import config
 
 #------------------------------------------------------------------------
 #
@@ -65,11 +70,11 @@ class SurnameCloudGramplet(Gramplet):
         self.set_text(_("No Family Tree loaded."))
 
     def db_changed(self):
-        self.dbstate.db.connect('person-add', self.update)
-        self.dbstate.db.connect('person-delete', self.update)
-        self.dbstate.db.connect('person-update', self.update)
-        self.dbstate.db.connect('person-rebuild', self.update)
-        self.dbstate.db.connect('family-rebuild', self.update)
+        self.connect(self.dbstate.db, 'person-add', self.update)
+        self.connect(self.dbstate.db, 'person-delete', self.update)
+        self.connect(self.dbstate.db, 'person-update', self.update)
+        self.connect(self.dbstate.db, 'person-rebuild', self.update)
+        self.connect(self.dbstate.db, 'family-rebuild', self.update)
 
     def on_load(self):
         if len(self.gui.data) == 3:
@@ -91,7 +96,7 @@ class SurnameCloudGramplet(Gramplet):
         representative_handle = {}
 
         cnt = 0
-        namelist =[]
+        namelist = []
         for person in self.dbstate.db.iter_people():
             allnames = [person.get_primary_name()] + person.get_alternate_names()
             allnames = set([name.get_group_name().strip() for name in allnames])
@@ -111,7 +116,7 @@ class SurnameCloudGramplet(Gramplet):
         surname_sort = []
         total = cnt = 0
         for surname in surnames:
-            surname_sort.append( (surnames[surname], surname) )
+            surname_sort.append((surnames[surname], surname))
             total += surnames[surname]
             cnt += 1
             if not cnt % _YIELD_INTERVAL:
@@ -121,10 +126,10 @@ class SurnameCloudGramplet(Gramplet):
         cloud_names = []
         cloud_values = []
         for (count, surname) in surname_sort:
-            cloud_names.append( (count, surname) )
-            cloud_values.append( count )
+            cloud_names.append((count, surname))
+            cloud_values.append(count)
 
-        cloud_names.sort(key=lambda k:k[1])
+        cloud_names.sort(key=lambda k: k[1])
         line = 0
         ### All done!
         # Now, find out how many we can display without going over top_size:
@@ -141,7 +146,7 @@ class SurnameCloudGramplet(Gramplet):
                 include_greater_than = s
                 break
         # now, limit counts to only include those that we can display:
-        
+
         mins = self.min_font
         maxs = self.max_font
         # Ok, now we can show those counts > include_greater_than:
@@ -161,12 +166,12 @@ class SurnameCloudGramplet(Gramplet):
                     text = surname
                 size = make_tag_size(count, counts, mins=mins, maxs=maxs)
                 self.link(text, 'Surname', representative_handle[surname], size,
-                          "%s, %d%% (%d)" % (text, 
-                                             int((float(count)/total_people) * 100), 
+                          "%s, %d%% (%d)" % (text,
+                                             int((float(count)/total_people) * 100),
                                              count))
                 self.append_text(" ")
                 showing += 1
-        self.append_text(("\n\n" + _("Total unique surnames") + ": %d\n") % 
+        self.append_text(("\n\n" + _("Total unique surnames") + ": %d\n") %
                          len(namelist))
         self.append_text((_("Total surnames showing") + ": %d\n") % showing)
         self.append_text((_("Total people") + ": %d") % total_people, "begin")
